firefox/firefox-pipewire.patch

2078 lines
78 KiB
Diff
Raw Normal View History

diff -up firefox-64.0/config/system-headers.mozbuild.firefox-pipewire firefox-64.0/config/system-headers.mozbuild
--- firefox-64.0/config/system-headers.mozbuild.firefox-pipewire 2018-12-04 00:34:00.000000000 +0100
+++ firefox-64.0/config/system-headers.mozbuild 2018-12-05 13:04:08.828063572 +0100
@@ -314,6 +314,7 @@ system_headers = [
'Gestalt.h',
'getopt.h',
'gio/gio.h',
+ 'gio/gunixfdlist.h',
'glibconfig.h',
'glib.h',
'glib-object.h',
@@ -607,6 +608,7 @@ system_headers = [
'Pgenerr.h',
'PGenErr.h',
'Ph.h',
+ 'pipewire/pipewire.h',
'pixman.h',
'pk11func.h',
'pk11pqg.h',
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_linux.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_linux.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_linux.cc.firefox-pipewire 2018-12-05 13:04:08.828063572 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_linux.cc 2018-12-05 13:04:08.828063572 +0100
@@ -0,0 +1,55 @@
+/*
+ * Copyright (c) 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
+#include "webrtc/modules/desktop_capture/desktop_capture_options.h"
+#include "webrtc/modules/desktop_capture/desktop_capturer.h"
+
+#if defined(USE_PIPEWIRE)
+#include "webrtc/modules/desktop_capture/app_capturer_null.cc"
+#endif // defined(USE_PIPEWIRE)
+
+#if defined(USE_X11)
+#include "webrtc/modules/desktop_capture/app_capturer_x11.cc"
+#endif // defined(USE_X11)
+
+namespace webrtc {
+
+// static
+AppCapturer* AppCapturer::Create(const DesktopCaptureOptions& options) {
+#if defined(USE_PIPEWIRE)
+ if (DesktopCapturer::IsRunningUnderWayland()) {
+ return new AppCapturerNull();
+ }
+#endif // defined(USE_PIPEWIRE)
+
+#if defined(USE_X11)
+ return AppCapturerX11::Create(options);
+#endif // defined(USE_X11)
+
+ return nullptr;
+}
+
+// static
+std::unique_ptr<DesktopCapturer> DesktopCapturer::CreateRawAppCapturer(
+ const DesktopCaptureOptions& options) {
+#if defined(USE_PIPEWIRE)
+ if (DesktopCapturer::IsRunningUnderWayland()) {
+ return std::unique_ptr<DesktopCapturer>(new AppCapturerNull());
+ }
+#endif // defined(USE_PIPEWIRE)
+
+#if defined(USE_X11)
+ return AppCapturerX11::CreateRawAppCapturer(options);
+#endif // defined(USE_X11)
+
+ return nullptr;
+}
+
+} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_null.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_null.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_null.cc.firefox-pipewire 2018-12-04 00:34:15.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_null.cc 2018-12-05 13:04:08.829063566 +0100
@@ -7,8 +7,8 @@
* in the file PATENTS. All contributing project authors may
* be found in the AUTHORS file in the root of the source tree.
*/
-#include "webrtc/modules/desktop_capture/window_capturer.h"
#include "webrtc/modules/desktop_capture/app_capturer.h"
+#include "webrtc/modules/desktop_capture/desktop_capturer.h"
2018-10-18 10:19:30 +00:00
#include <assert.h>
2018-10-18 10:19:30 +00:00
@@ -29,9 +29,11 @@ public:
virtual bool BringAppToFront() override;
2018-10-18 10:19:30 +00:00
// DesktopCapturer interface.
- virtual void Start(Callback* callback) override;
- virtual void Stop() override;
- virtual void Capture(const DesktopRegion& region) override;
+ void Start(Callback* callback) override;
+ void Stop() override;
+ void CaptureFrame() override;
+ bool GetSourceList(SourceList* sources) override;
+ bool SelectSource(SourceId id) override;
2018-10-18 10:19:30 +00:00
private:
Callback* callback_;
@@ -61,7 +63,16 @@ bool AppCapturerNull::BringAppToFront()
return false;
}
2018-10-18 10:19:30 +00:00
-// DesktopCapturer interface.
+bool AppCapturerNull::GetSourceList(SourceList* sources) {
+ // Not implemented yet.
+ return false;
+}
+
+bool AppCapturerNull::SelectSource(SourceId id) {
+ // Not implemented yet.
+ return false;
+}
+
void AppCapturerNull::Start(Callback* callback) {
assert(!callback_);
assert(callback);
@@ -73,16 +84,19 @@ void AppCapturerNull::Stop() {
callback_ = NULL;
}
2018-10-18 10:19:30 +00:00
-void AppCapturerNull::Capture(const DesktopRegion& region) {
- // Not implemented yet: See Bug 1036653
- callback_->OnCaptureCompleted(NULL);
+void AppCapturerNull::CaptureFrame() {
+ // Not implemented yet.
+ callback_->OnCaptureResult(Result::ERROR_TEMPORARY, nullptr);
}
2018-10-18 10:19:30 +00:00
} // namespace
2018-10-18 10:19:30 +00:00
+#if not defined(USE_PIPEWIRE)
// static
-AppCapturer* AppCapturer::Create(const DesktopCaptureOptions& options) {
- return new AppCapturerNull();
+std::unique_ptr<DesktopCapturer> DesktopCapturer::CreateRawAppCapturer(
+ const DesktopCaptureOptions& options) {
+ return std::unique_ptr<DesktopCapturer>(new AppCapturerNull());
}
+#endif // not defined(USE_PIPEWIRE)
2018-10-18 10:19:30 +00:00
} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_x11.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_x11.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_x11.cc.firefox-pipewire 2018-12-04 00:34:14.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_x11.cc 2018-12-05 13:04:08.829063566 +0100
@@ -53,10 +53,13 @@ protected:
std::unique_ptr<DesktopFrame> frame_;
};
2018-10-18 10:19:30 +00:00
-class AppCapturerLinux : public AppCapturer {
+class AppCapturerX11 : public AppCapturer {
public:
- AppCapturerLinux(const DesktopCaptureOptions& options);
- virtual ~AppCapturerLinux();
+ AppCapturerX11(const DesktopCaptureOptions& options);
+ virtual ~AppCapturerX11();
+
+ static AppCapturer* Create(const DesktopCaptureOptions& options);
+ static std::unique_ptr<DesktopCapturer> CreateRawAppCapturer(const DesktopCaptureOptions& options);
2018-10-18 10:19:30 +00:00
// AppCapturer interface.
virtual bool GetAppList(AppList* apps) override;
@@ -91,10 +94,9 @@ private:
Region rgn_background_;
2018-10-18 10:19:30 +00:00
rtc::scoped_refptr<SharedXDisplay> x_display_;
- RTC_DISALLOW_COPY_AND_ASSIGN(AppCapturerLinux);
+ RTC_DISALLOW_COPY_AND_ASSIGN(AppCapturerX11);
};
-
-AppCapturerLinux::AppCapturerLinux(const DesktopCaptureOptions& options)
+AppCapturerX11::AppCapturerX11(const DesktopCaptureOptions& options)
: callback_(NULL),
selected_process_(0),
x_display_(options.x_display()) {
@@ -103,7 +105,7 @@ AppCapturerLinux::AppCapturerLinux(const
rgn_background_ = XCreateRegion();
}
2018-10-18 10:19:30 +00:00
-AppCapturerLinux::~AppCapturerLinux() {
+AppCapturerX11::~AppCapturerX11() {
if (rgn_mask_) {
XDestroyRegion(rgn_mask_);
}
@@ -116,32 +118,32 @@ AppCapturerLinux::~AppCapturerLinux() {
}
2018-10-18 10:19:30 +00:00
// AppCapturer interface.
-bool AppCapturerLinux::GetAppList(AppList* apps) {
+bool AppCapturerX11::GetAppList(AppList* apps) {
// Implemented in DesktopDeviceInfo
return true;
}
-bool AppCapturerLinux::SelectApp(ProcessId processId) {
+bool AppCapturerX11::SelectApp(ProcessId processId) {
selected_process_ = processId;
return true;
}
-bool AppCapturerLinux::BringAppToFront() {
+bool AppCapturerX11::BringAppToFront() {
// Not implemented yet: See Bug 1036653
return true;
}
2018-10-18 10:19:30 +00:00
// DesktopCapturer interface.
-void AppCapturerLinux::Start(Callback* callback) {
+void AppCapturerX11::Start(Callback* callback) {
assert(!callback_);
assert(callback);
2018-10-18 10:19:30 +00:00
callback_ = callback;
}
2018-10-18 10:19:30 +00:00
-void AppCapturerLinux::Stop() {
+void AppCapturerX11::Stop() {
callback_ = NULL;
}
2018-10-18 10:19:30 +00:00
-void AppCapturerLinux::CaptureFrame() {
+void AppCapturerX11::CaptureFrame() {
XErrorTrap error_trap(GetDisplay());
2018-10-18 10:19:30 +00:00
//Capture screen >> set root window as capture window
@@ -169,7 +171,7 @@ void AppCapturerLinux::CaptureFrame() {
}
}
2018-10-18 10:19:30 +00:00
-void AppCapturerLinux::FillDesktopFrameRegionWithColor(DesktopFrame* pDesktopFrame, Region rgn, uint32_t color) {
+void AppCapturerX11::FillDesktopFrameRegionWithColor(DesktopFrame* pDesktopFrame, Region rgn, uint32_t color) {
XErrorTrap error_trap(GetDisplay());
2018-10-18 10:19:30 +00:00
if (!pDesktopFrame) {
@@ -192,7 +194,7 @@ void AppCapturerLinux::FillDesktopFrameR
}
}
2018-10-18 10:19:30 +00:00
-bool AppCapturerLinux::UpdateRegions() {
+bool AppCapturerX11::UpdateRegions() {
XErrorTrap error_trap(GetDisplay());
2018-10-18 10:19:30 +00:00
XSubtractRegion(rgn_visual_, rgn_visual_, rgn_visual_);
@@ -272,18 +274,18 @@ bool AppCapturerLinux::UpdateRegions() {
} // namespace
2018-10-18 10:19:30 +00:00
// static
-AppCapturer* AppCapturer::Create(const DesktopCaptureOptions& options) {
- return new AppCapturerLinux(options);
+AppCapturer* AppCapturerX11::Create(const DesktopCaptureOptions& options) {
+ return new AppCapturerX11(options);
}
2018-10-18 10:19:30 +00:00
// static
-std::unique_ptr<DesktopCapturer> DesktopCapturer::CreateRawAppCapturer(
+std::unique_ptr<DesktopCapturer> AppCapturerX11::CreateRawAppCapturer(
const DesktopCaptureOptions& options) {
2018-10-18 10:19:30 +00:00
if (!options.x_display())
return nullptr;
2018-10-18 10:19:30 +00:00
- std::unique_ptr<AppCapturerLinux> capturer(new AppCapturerLinux(options));
+ std::unique_ptr<AppCapturerX11> capturer(new AppCapturerX11(options));
2018-10-18 10:19:30 +00:00
return std::unique_ptr<DesktopCapturer>(std::move(capturer));
}
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.cc.firefox-pipewire 2018-12-05 13:04:08.832063551 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.cc 2018-12-05 13:04:08.832063551 +0100
2018-10-18 10:19:30 +00:00
@@ -0,0 +1,849 @@
+/*
2018-10-18 10:19:30 +00:00
+ * Copyright 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
2018-10-18 10:19:30 +00:00
+#include <cstring>
+#include <gio/gunixfdlist.h>
+#include <glib-object.h>
+
2018-10-18 10:19:30 +00:00
+#include <spa/param/format-utils.h>
+#include <spa/param/props.h>
+#include <spa/param/video/raw-utils.h>
+#include <spa/support/type-map.h>
+
2018-10-18 10:19:30 +00:00
+#include "webrtc/modules/desktop_capture/base_capturer_pipewire.h"
+
+#include "webrtc/modules/desktop_capture/desktop_capture_options.h"
+#include "webrtc/modules/desktop_capture/desktop_capturer.h"
+#include "webrtc/base/checks.h"
+#include "webrtc/base/logging.h"
+
+namespace webrtc {
+
2018-10-18 10:19:30 +00:00
+const char kDesktopBusName[] = "org.freedesktop.portal.Desktop";
+const char kDesktopObjectPath[] = "/org/freedesktop/portal/desktop";
+const char kDesktopRequestObjectPath[] =
+ "/org/freedesktop/portal/desktop/request";
+const char kSessionInterfaceName[] = "org.freedesktop.portal.Session";
+const char kRequestInterfaceName[] = "org.freedesktop.portal.Request";
+const char kScreenCastInterfaceName[] = "org.freedesktop.portal.ScreenCast";
+
+const int kBytesPerPixelPw = 4;
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnStateChanged(void* data,
+ pw_remote_state old_state,
+ pw_remote_state state,
+ const char* error_message) {
+ BaseCapturerPipeWire* that = static_cast<BaseCapturerPipeWire*>(data);
+ RTC_DCHECK(that);
+
+ switch (state) {
+ case PW_REMOTE_STATE_ERROR:
+ LOG(LS_ERROR) << "PipeWire remote state error: " << error_message;
+ break;
+ case PW_REMOTE_STATE_CONNECTED:
+ LOG(LS_INFO) << "PipeWire remote state: connected.";
+ that->CreateReceivingStream();
+ break;
+ case PW_REMOTE_STATE_CONNECTING:
+ LOG(LS_INFO) << "PipeWire remote state: connecting.";
+ break;
+ case PW_REMOTE_STATE_UNCONNECTED:
+ LOG(LS_INFO) << "PipeWire remote state: unconnected.";
+ break;
+ }
+}
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnStreamStateChanged(void* data,
+ pw_stream_state old_state,
+ pw_stream_state state,
+ const char* error_message) {
+ BaseCapturerPipeWire* that = static_cast<BaseCapturerPipeWire*>(data);
+ RTC_DCHECK(that);
+
+ switch (state) {
+ case PW_STREAM_STATE_ERROR:
+ LOG(LS_ERROR) << "PipeWire stream state error: " << error_message;
+ break;
+ case PW_STREAM_STATE_CONFIGURE:
+ pw_stream_set_active(that->pw_stream_, true);
+ break;
+ case PW_STREAM_STATE_UNCONNECTED:
+ case PW_STREAM_STATE_CONNECTING:
+ case PW_STREAM_STATE_READY:
+ case PW_STREAM_STATE_PAUSED:
+ case PW_STREAM_STATE_STREAMING:
+ break;
+ }
+}
+
2018-10-18 10:19:30 +00:00
+// static
+void BaseCapturerPipeWire::OnStreamFormatChanged(
+ void* data,
+ const struct spa_pod* format) {
+ BaseCapturerPipeWire* that = static_cast<BaseCapturerPipeWire*>(data);
+ RTC_DCHECK(that);
+
2018-10-18 10:19:30 +00:00
+ LOG(LS_INFO) << "PipeWire stream format changed.";
+
2018-10-18 10:19:30 +00:00
+ if (!format) {
+ pw_stream_finish_format(that->pw_stream_, /*res=*/0, /*params=*/nullptr,
+ /*n_params=*/0);
+ return;
+ }
+
2018-10-18 10:19:30 +00:00
+ that->spa_video_format_ = new spa_video_info_raw();
+ spa_format_video_raw_parse(format, that->spa_video_format_,
+ &that->pw_type_->format_video);
+
2018-10-18 10:19:30 +00:00
+ auto width = that->spa_video_format_->size.width;
+ auto height = that->spa_video_format_->size.height;
+ auto stride = SPA_ROUND_UP_N(width * kBytesPerPixelPw, 4);
+ auto size = height * stride;
+
2018-10-18 10:19:30 +00:00
+ uint8_t buffer[1024] = {};
+ auto builder = spa_pod_builder{buffer, sizeof(buffer)};
+
2018-10-18 10:19:30 +00:00
+ // Setup buffers and meta header for new format.
+ const struct spa_pod* params[2];
+ params[0] = reinterpret_cast<spa_pod*>(spa_pod_builder_object(
+ &builder,
+ // id to enumerate buffer requirements
+ that->pw_core_type_->param.idBuffers,
+ that->pw_core_type_->param_buffers.Buffers,
+ // Size: specified as integer (i) and set to specified size
+ ":", that->pw_core_type_->param_buffers.size, "i", size,
+ // Stride: specified as integer (i) and set to specified stride
+ ":", that->pw_core_type_->param_buffers.stride, "i", stride,
+ // Buffers: specifies how many buffers we want to deal with, set as
+ // integer (i) where preferred number is 8, then allowed number is defined
+ // as range (r) from min and max values and it is undecided (u) to allow
+ // negotiation
+ ":", that->pw_core_type_->param_buffers.buffers, "iru", 8,
+ SPA_POD_PROP_MIN_MAX(1, 32),
+ // Align: memory alignment of the buffer, set as integer (i) to specified
+ // value
+ ":", that->pw_core_type_->param_buffers.align, "i", 16));
+ params[1] = reinterpret_cast<spa_pod*>(spa_pod_builder_object(
+ &builder,
+ // id to enumerate supported metadata
+ that->pw_core_type_->param.idMeta, that->pw_core_type_->param_meta.Meta,
+ // Type: specified as id or enum (I)
+ ":", that->pw_core_type_->param_meta.type, "I",
+ that->pw_core_type_->meta.Header,
+ // Size: size of the metadata, specified as integer (i)
+ ":", that->pw_core_type_->param_meta.size, "i",
+ sizeof(struct spa_meta_header)));
+
+ pw_stream_finish_format(that->pw_stream_, /*res=*/0, params, /*n_params=*/2);
+}
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnStreamProcess(void* data) {
+ BaseCapturerPipeWire* that = static_cast<BaseCapturerPipeWire*>(data);
+ RTC_DCHECK(that);
+
+ pw_buffer* buf = nullptr;
+
+ if (!(buf = pw_stream_dequeue_buffer(that->pw_stream_))) {
+ return;
+ }
+
2018-10-18 10:19:30 +00:00
+ that->HandleBuffer(buf);
+
2018-10-18 10:19:30 +00:00
+ pw_stream_queue_buffer(that->pw_stream_, buf);
+}
+
2018-10-18 10:19:30 +00:00
+BaseCapturerPipeWire::BaseCapturerPipeWire(CaptureSourceType source_type)
+ : capture_source_type_(source_type) {}
+
2018-10-18 10:19:30 +00:00
+BaseCapturerPipeWire::~BaseCapturerPipeWire() {
+ if (pw_main_loop_) {
+ pw_thread_loop_stop(pw_main_loop_);
+ }
+
2018-10-18 10:19:30 +00:00
+ if (pw_type_) {
+ delete pw_type_;
+ }
+
2018-10-18 10:19:30 +00:00
+ if (spa_video_format_) {
+ delete spa_video_format_;
+ }
+
2018-10-18 10:19:30 +00:00
+ if (pw_stream_) {
+ pw_stream_destroy(pw_stream_);
+ }
+
2018-10-18 10:19:30 +00:00
+ if (pw_remote_) {
+ pw_remote_destroy(pw_remote_);
+ }
+
2018-10-18 10:19:30 +00:00
+ if (pw_core_) {
+ pw_core_destroy(pw_core_);
+ }
+
2018-10-18 10:19:30 +00:00
+ if (pw_main_loop_) {
+ pw_thread_loop_destroy(pw_main_loop_);
+ }
+
2018-10-18 10:19:30 +00:00
+ if (pw_loop_) {
+ pw_loop_destroy(pw_loop_);
+ }
+
2018-10-18 10:19:30 +00:00
+ if (current_frame_) {
+ free(current_frame_);
+ }
+
2018-10-18 10:19:30 +00:00
+ if (start_request_signal_id_) {
+ g_dbus_connection_signal_unsubscribe(connection_, start_request_signal_id_);
+ }
+ if (sources_request_signal_id_) {
+ g_dbus_connection_signal_unsubscribe(connection_,
+ sources_request_signal_id_);
+ }
+ if (session_request_signal_id_) {
+ g_dbus_connection_signal_unsubscribe(connection_,
+ session_request_signal_id_);
+ }
+
2018-10-18 10:19:30 +00:00
+ if (session_handle_) {
+ GDBusMessage* message = g_dbus_message_new_method_call(
+ kDesktopBusName, session_handle_, kSessionInterfaceName, "Close");
+ if (message) {
+ GError* error = nullptr;
+ g_dbus_connection_send_message(connection_, message,
+ G_DBUS_SEND_MESSAGE_FLAGS_NONE,
+ /*out_serial=*/nullptr, &error);
+ if (error) {
+ LOG(LS_ERROR) << "Failed to close the session: " << error->message;
+ g_error_free(error);
+ }
+ g_object_unref(message);
+ }
+ }
+
2018-10-18 10:19:30 +00:00
+ g_free(start_handle_);
+ g_free(sources_handle_);
+ g_free(session_handle_);
+ g_free(portal_handle_);
+
2018-10-18 10:19:30 +00:00
+ if (proxy_) {
+ g_clear_object(&proxy_);
+ }
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::InitPortals() {
+ g_dbus_proxy_new_for_bus(
+ G_BUS_TYPE_SESSION, G_DBUS_PROXY_FLAGS_NONE, /*info=*/nullptr,
+ kDesktopBusName, kDesktopObjectPath, kScreenCastInterfaceName,
+ /*cancellable=*/nullptr,
+ reinterpret_cast<GAsyncReadyCallback>(OnProxyRequested), this);
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::InitPipeWire() {
+ pw_init(/*argc=*/nullptr, /*argc=*/nullptr);
+
2018-10-18 10:19:30 +00:00
+ pw_loop_ = pw_loop_new(/*properties=*/nullptr);
+ pw_main_loop_ = pw_thread_loop_new(pw_loop_, "pipewire-main-loop");
+
2018-10-18 10:19:30 +00:00
+ pw_core_ = pw_core_new(pw_loop_, /*properties=*/nullptr);
+ pw_core_type_ = pw_core_get_type(pw_core_);
+ pw_remote_ = pw_remote_new(pw_core_, nullptr, /*user_data_size=*/0);
+
2018-10-18 10:19:30 +00:00
+ InitPipeWireTypes();
+
2018-10-18 10:19:30 +00:00
+ // Initialize event handlers, remote end and stream-related.
+ pw_remote_events_.version = PW_VERSION_REMOTE_EVENTS;
+ pw_remote_events_.state_changed = &OnStateChanged;
+
2018-10-18 10:19:30 +00:00
+ pw_stream_events_.version = PW_VERSION_STREAM_EVENTS;
+ pw_stream_events_.state_changed = &OnStreamStateChanged;
+ pw_stream_events_.format_changed = &OnStreamFormatChanged;
+ pw_stream_events_.process = &OnStreamProcess;
+
2018-10-18 10:19:30 +00:00
+ pw_remote_add_listener(pw_remote_, &spa_remote_listener_, &pw_remote_events_,
+ this);
+ pw_remote_connect_fd(pw_remote_, pw_fd_);
+
2018-10-18 10:19:30 +00:00
+ if (pw_thread_loop_start(pw_main_loop_) < 0) {
+ LOG(LS_ERROR) << "Failed to start main PipeWire loop";
+ portal_init_failed_ = true;
+ }
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::InitPipeWireTypes() {
+ spa_type_map* map = pw_core_type_->map;
+ pw_type_ = new PipeWireType();
+
2018-10-18 10:19:30 +00:00
+ spa_type_media_type_map(map, &pw_type_->media_type);
+ spa_type_media_subtype_map(map, &pw_type_->media_subtype);
+ spa_type_format_video_map(map, &pw_type_->format_video);
+ spa_type_video_format_map(map, &pw_type_->video_format);
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::CreateReceivingStream() {
+ spa_rectangle pwMinScreenBounds = spa_rectangle{1, 1};
+ spa_rectangle pwScreenBounds =
+ spa_rectangle{static_cast<uint32_t>(desktop_size_.width()),
+ static_cast<uint32_t>(desktop_size_.height())};
+
2018-10-18 10:19:30 +00:00
+ spa_fraction pwFrameRateMin = spa_fraction{0, 1};
+ spa_fraction pwFrameRateMax = spa_fraction{60, 1};
+
2018-10-18 10:19:30 +00:00
+ pw_properties* reuseProps = pw_properties_new("pipewire.client.reuse", "1",
+ /*end of varargs*/ nullptr);
+ pw_stream_ = pw_stream_new(pw_remote_, "webrtc-consume-stream", reuseProps);
+
2018-10-18 10:19:30 +00:00
+ uint8_t buffer[1024] = {};
+ const spa_pod* params[1];
+ spa_pod_builder builder = spa_pod_builder{buffer, sizeof(buffer)};
+ params[0] = reinterpret_cast<spa_pod*>(spa_pod_builder_object(
+ &builder,
+ // id to enumerate formats
+ pw_core_type_->param.idEnumFormat, pw_core_type_->spa_format, "I",
+ pw_type_->media_type.video, "I", pw_type_->media_subtype.raw,
+ // Video format: specified as id or enum (I), preferred format is BGRx,
+ // then allowed formats are enumerated (e) and the format is undecided (u)
+ // to allow negotiation
+ ":", pw_type_->format_video.format, "Ieu", pw_type_->video_format.BGRx,
+ SPA_POD_PROP_ENUM(2, pw_type_->video_format.RGBx,
+ pw_type_->video_format.BGRx),
+ // Video size: specified as rectangle (R), preferred size is specified as
+ // first parameter, then allowed size is defined as range (r) from min and
+ // max values and the format is undecided (u) to allow negotiation
+ ":", pw_type_->format_video.size, "Rru", &pwScreenBounds, 2,
+ &pwMinScreenBounds, &pwScreenBounds,
+ // Frame rate: specified as fraction (F) and set to minimum frame rate
+ // value
+ ":", pw_type_->format_video.framerate, "F", &pwFrameRateMin,
+ // Max frame rate: specified as fraction (F), preferred frame rate is set
+ // to maximum value, then allowed frame rate is defined as range (r) from
+ // min and max values and it is undecided (u) to allow negotiation
+ ":", pw_type_->format_video.max_framerate, "Fru", &pwFrameRateMax, 2,
+ &pwFrameRateMin, &pwFrameRateMax));
+
2018-10-18 10:19:30 +00:00
+ pw_stream_add_listener(pw_stream_, &spa_stream_listener_, &pw_stream_events_,
+ this);
+ pw_stream_flags flags = static_cast<pw_stream_flags>(
+ PW_STREAM_FLAG_AUTOCONNECT | PW_STREAM_FLAG_INACTIVE |
+ PW_STREAM_FLAG_MAP_BUFFERS);
+ if (pw_stream_connect(pw_stream_, PW_DIRECTION_INPUT, /*port_path=*/nullptr,
+ flags, params,
+ /*n_params=*/1) != 0) {
+ LOG(LS_ERROR) << "Could not connect receiving stream.";
+ portal_init_failed_ = true;
+ return;
+ }
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::HandleBuffer(pw_buffer* buffer) {
+ spa_buffer* spaBuffer = buffer->buffer;
+ void* src = nullptr;
+
2018-10-18 10:19:30 +00:00
+ if (!(src = spaBuffer->datas[0].data)) {
+ return;
+ }
+
2018-10-18 10:19:30 +00:00
+ uint32_t maxSize = spaBuffer->datas[0].maxsize;
+ int32_t srcStride = spaBuffer->datas[0].chunk->stride;
+ if (srcStride != (desktop_size_.width() * kBytesPerPixelPw)) {
+ LOG(LS_ERROR) << "Got buffer with stride different from screen stride: "
+ << srcStride
+ << " != " << (desktop_size_.width() * kBytesPerPixelPw);
+ portal_init_failed_ = true;
+ return;
+ }
+
2018-10-18 10:19:30 +00:00
+ if (!current_frame_) {
+ current_frame_ = static_cast<uint8_t*>(malloc(maxSize));
+ }
+ RTC_DCHECK(current_frame_ != nullptr);
+
2018-10-18 10:19:30 +00:00
+ // If both sides decided to go with the RGBx format we need to convert it to
+ // BGRx to match color format expected by WebRTC.
+ if (spa_video_format_->format == pw_type_->video_format.RGBx) {
+ uint8_t* tempFrame = static_cast<uint8_t*>(malloc(maxSize));
+ std::memcpy(tempFrame, src, maxSize);
+ ConvertRGBxToBGRx(tempFrame, maxSize);
+ std::memcpy(current_frame_, tempFrame, maxSize);
+ free(tempFrame);
+ } else {
+ std::memcpy(current_frame_, src, maxSize);
+ }
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::ConvertRGBxToBGRx(uint8_t* frame, uint32_t size) {
+ // Change color format for KDE KWin which uses RGBx and not BGRx
+ for (uint32_t i = 0; i < size; i += 4) {
+ uint8_t tempR = frame[i];
+ uint8_t tempB = frame[i + 2];
+ frame[i] = tempB;
+ frame[i + 2] = tempR;
+ }
+}
+
2018-10-18 10:19:30 +00:00
+guint BaseCapturerPipeWire::SetupRequestResponseSignal(
+ const gchar* object_path,
+ GDBusSignalCallback callback) {
+ return g_dbus_connection_signal_subscribe(
+ connection_, kDesktopBusName, kRequestInterfaceName, "Response",
+ object_path, /*arg0=*/nullptr, G_DBUS_SIGNAL_FLAGS_NO_MATCH_RULE,
+ callback, this, /*user_data_free_func=*/nullptr);
+}
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnProxyRequested(GObject* /*object*/,
+ GAsyncResult* result,
+ gpointer user_data) {
+ BaseCapturerPipeWire* that =
+ static_cast<BaseCapturerPipeWire*>(user_data);
+ RTC_DCHECK(that);
+
2018-10-18 10:19:30 +00:00
+ GError* error = nullptr;
+ that->proxy_ = g_dbus_proxy_new_finish(result, &error);
+ if (!that->proxy_) {
+ LOG(LS_ERROR) << "Failed to create a proxy for the screen cast portal: "
+ << error->message;
+ g_error_free(error);
+ that->portal_init_failed_ = true;
+ return;
+ }
2018-10-18 10:19:30 +00:00
+ that->connection_ = g_dbus_proxy_get_connection(that->proxy_);
+
2018-10-18 10:19:30 +00:00
+ LOG(LS_INFO) << "Created proxy for the screen cast portal.";
+ that->SessionRequest();
+}
+
2018-10-18 10:19:30 +00:00
+// static
+gchar* BaseCapturerPipeWire::PrepareSignalHandle(GDBusConnection* connection,
+ const gchar* token) {
+ gchar* sender = g_strdup(g_dbus_connection_get_unique_name(connection) + 1);
+ for (int i = 0; sender[i]; i++) {
+ if (sender[i] == '.') {
+ sender[i] = '_';
+ }
+ }
+
2018-10-18 10:19:30 +00:00
+ gchar* handle = g_strconcat(kDesktopRequestObjectPath, "/", sender, "/",
+ token, /*end of varargs*/ nullptr);
+ g_free(sender);
+
2018-10-18 10:19:30 +00:00
+ return handle;
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::SessionRequest() {
+ GVariantBuilder builder;
+ gchar* variant_string;
+
+ g_variant_builder_init(&builder, G_VARIANT_TYPE_VARDICT);
+ variant_string =
+ g_strdup_printf("webrtc_session%d", g_random_int_range(0, G_MAXINT));
+ g_variant_builder_add(&builder, "{sv}", "session_handle_token",
+ g_variant_new_string(variant_string));
+ g_free(variant_string);
+ variant_string = g_strdup_printf("webrtc%d", g_random_int_range(0, G_MAXINT));
+ g_variant_builder_add(&builder, "{sv}", "handle_token",
+ g_variant_new_string(variant_string));
+
+ portal_handle_ = PrepareSignalHandle(connection_, variant_string);
+ session_request_signal_id_ = SetupRequestResponseSignal(
+ portal_handle_, OnSessionRequestResponseSignal);
+ g_free(variant_string);
+
+ LOG(LS_INFO) << "Screen cast session requested.";
+ g_dbus_proxy_call(
+ proxy_, "CreateSession", g_variant_new("(a{sv})", &builder),
+ G_DBUS_CALL_FLAGS_NONE, /*timeout=*/-1, /*cancellable=*/nullptr,
+ reinterpret_cast<GAsyncReadyCallback>(OnSessionRequested), this);
+}
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnSessionRequested(GDBusConnection* connection,
+ GAsyncResult* result,
+ gpointer user_data) {
+ BaseCapturerPipeWire* that =
+ static_cast<BaseCapturerPipeWire*>(user_data);
+ RTC_DCHECK(that);
+
2018-10-18 10:19:30 +00:00
+ GError* error = nullptr;
+ GVariant* variant = g_dbus_proxy_call_finish(that->proxy_, result, &error);
+ if (!variant) {
+ LOG(LS_ERROR) << "Failed to create a screen cast session: "
+ << error->message;
+ g_error_free(error);
+ that->portal_init_failed_ = true;
+ return;
+ }
+ LOG(LS_INFO) << "Initializing the screen cast session.";
+
2018-10-18 10:19:30 +00:00
+ gchar* handle = nullptr;
+ g_variant_get_child(variant, 0, "o", &handle);
+ g_variant_unref(variant);
+ if (!handle) {
+ LOG(LS_ERROR) << "Failed to initialize the screen cast session.";
+ if (that->session_request_signal_id_) {
+ g_dbus_connection_signal_unsubscribe(connection,
+ that->session_request_signal_id_);
+ that->session_request_signal_id_ = 0;
+ }
+ that->portal_init_failed_ = true;
+ return;
+ }
+
2018-10-18 10:19:30 +00:00
+ g_free(handle);
+
2018-10-18 10:19:30 +00:00
+ LOG(LS_INFO) << "Subscribing to the screen cast session.";
+}
+
2018-10-18 10:19:30 +00:00
+// static
+void BaseCapturerPipeWire::OnSessionRequestResponseSignal(
+ GDBusConnection* connection,
+ const gchar* sender_name,
+ const gchar* object_path,
+ const gchar* interface_name,
+ const gchar* signal_name,
+ GVariant* parameters,
+ gpointer user_data) {
+ BaseCapturerPipeWire* that =
+ static_cast<BaseCapturerPipeWire*>(user_data);
+ RTC_DCHECK(that);
+
+
2018-10-18 10:19:30 +00:00
+ LOG(LS_INFO)
+ << "Received response for the screen cast session subscription.";
+
2018-10-18 10:19:30 +00:00
+ guint32 portal_response;
+ GVariant* response_data;
+ g_variant_get(parameters, "(u@a{sv})", &portal_response, &response_data);
+ g_variant_lookup(response_data, "session_handle", "s",
+ &that->session_handle_);
+ g_variant_unref(response_data);
+
2018-10-18 10:19:30 +00:00
+ if (!that->session_handle_ || portal_response) {
+ LOG(LS_ERROR)
+ << "Failed to request the screen cast session subscription.";
+ that->portal_init_failed_ = true;
+ return;
+ }
+
2018-10-18 10:19:30 +00:00
+ that->SourcesRequest();
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::SourcesRequest() {
+ GVariantBuilder builder;
+ gchar* variant_string;
+
+ g_variant_builder_init(&builder, G_VARIANT_TYPE_VARDICT);
2018-10-18 10:19:30 +00:00
+ // We want to record monitor content.
+ g_variant_builder_add(&builder, "{sv}", "types",
+ g_variant_new_uint32(capture_source_type_));
+ // We don't want to allow selection of multiple sources.
+ g_variant_builder_add(&builder, "{sv}", "multiple",
+ g_variant_new_boolean(false));
+ variant_string = g_strdup_printf("webrtc%d", g_random_int_range(0, G_MAXINT));
+ g_variant_builder_add(&builder, "{sv}", "handle_token",
+ g_variant_new_string(variant_string));
+
2018-10-18 10:19:30 +00:00
+ sources_handle_ = PrepareSignalHandle(connection_, variant_string);
+ sources_request_signal_id_ = SetupRequestResponseSignal(
+ sources_handle_, OnSourcesRequestResponseSignal);
+ g_free(variant_string);
+
2018-10-18 10:19:30 +00:00
+ LOG(LS_INFO) << "Requesting sources from the screen cast session.";
+ g_dbus_proxy_call(
2018-10-18 10:19:30 +00:00
+ proxy_, "SelectSources",
+ g_variant_new("(oa{sv})", session_handle_, &builder),
+ G_DBUS_CALL_FLAGS_NONE, /*timeout=*/-1, /*cancellable=*/nullptr,
2018-10-18 10:19:30 +00:00
+ reinterpret_cast<GAsyncReadyCallback>(OnSourcesRequested), this);
+}
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnSourcesRequested(GDBusConnection* connection,
+ GAsyncResult* result,
+ gpointer user_data) {
2018-10-18 10:19:30 +00:00
+ BaseCapturerPipeWire* that =
+ static_cast<BaseCapturerPipeWire*>(user_data);
+ RTC_DCHECK(that);
+
+ GError* error = nullptr;
+ GVariant* variant = g_dbus_proxy_call_finish(that->proxy_, result, &error);
+ if (!variant) {
2018-10-18 10:19:30 +00:00
+ LOG(LS_ERROR) << "Failed to request the sources: " << error->message;
+ g_error_free(error);
2018-10-18 10:19:30 +00:00
+ that->portal_init_failed_ = true;
+ return;
+ }
2018-10-18 10:19:30 +00:00
+
+ LOG(LS_INFO) << "Sources requested from the screen cast session.";
+
+ gchar* handle = nullptr;
+ g_variant_get_child(variant, 0, "o", &handle);
+ g_variant_unref(variant);
+ if (!handle) {
+ LOG(LS_ERROR) << "Failed to initialize the screen cast session.";
2018-10-18 10:19:30 +00:00
+ if (that->sources_request_signal_id_) {
+ g_dbus_connection_signal_unsubscribe(connection,
2018-10-18 10:19:30 +00:00
+ that->sources_request_signal_id_);
+ that->sources_request_signal_id_ = 0;
+ }
2018-10-18 10:19:30 +00:00
+ that->portal_init_failed_ = true;
+ return;
+ }
+
+ g_free(handle);
+
2018-10-18 10:19:30 +00:00
+ LOG(LS_INFO) << "Subscribed to sources signal.";
+}
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnSourcesRequestResponseSignal(
+ GDBusConnection* connection,
+ const gchar* sender_name,
+ const gchar* object_path,
+ const gchar* interface_name,
+ const gchar* signal_name,
+ GVariant* parameters,
+ gpointer user_data) {
2018-10-18 10:19:30 +00:00
+ BaseCapturerPipeWire* that =
+ static_cast<BaseCapturerPipeWire*>(user_data);
+ RTC_DCHECK(that);
+
+ guint32 portal_response;
2018-10-18 10:19:30 +00:00
+ g_variant_get(parameters, "(u@a{sv})", &portal_response, nullptr);
+ if (portal_response) {
+ LOG(LS_ERROR)
+ << "Failed to select sources for the screen cast session.";
+ that->portal_init_failed_ = true;
+ return;
+ }
+
2018-10-18 10:19:30 +00:00
+ LOG(LS_INFO) << "Received sources signal from session.";
+ that->StartRequest();
+}
+
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::StartRequest() {
+ GVariantBuilder builder;
+ gchar* variant_string;
+
+ g_variant_builder_init(&builder, G_VARIANT_TYPE_VARDICT);
+ variant_string = g_strdup_printf("webrtc%d", g_random_int_range(0, G_MAXINT));
+ g_variant_builder_add(&builder, "{sv}", "handle_token",
+ g_variant_new_string(variant_string));
+
+ start_handle_ = PrepareSignalHandle(connection_, variant_string);
+ start_request_signal_id_ =
+ SetupRequestResponseSignal(start_handle_, OnStartRequestResponseSignal);
+ g_free(variant_string);
+
+ // "Identifier for the application window", this is Wayland, so not "x11:...".
+ const gchar parent_window[] = "";
+
+ LOG(LS_INFO) << "Starting the screen cast session.";
+ g_dbus_proxy_call(
+ proxy_, "Start",
+ g_variant_new("(osa{sv})", session_handle_, parent_window, &builder),
+ G_DBUS_CALL_FLAGS_NONE, /*timeout=*/-1, /*cancellable=*/nullptr,
+ reinterpret_cast<GAsyncReadyCallback>(OnStartRequested), this);
+}
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnStartRequested(GDBusConnection* connection,
+ GAsyncResult* result,
+ gpointer user_data) {
2018-10-18 10:19:30 +00:00
+ BaseCapturerPipeWire* that =
+ static_cast<BaseCapturerPipeWire*>(user_data);
+ RTC_DCHECK(that);
+
+ GError* error = nullptr;
+ GVariant* variant = g_dbus_proxy_call_finish(that->proxy_, result, &error);
+ if (!variant) {
+ LOG(LS_ERROR) << "Failed to start the screen cast session: "
2018-10-18 10:19:30 +00:00
+ << error->message;
+ g_error_free(error);
2018-10-18 10:19:30 +00:00
+ that->portal_init_failed_ = true;
+ return;
+ }
+
+ LOG(LS_INFO) << "Initializing the start of the screen cast session.";
+
+ gchar* handle = nullptr;
+ g_variant_get_child(variant, 0, "o", &handle);
+ g_variant_unref(variant);
+ if (!handle) {
+ LOG(LS_ERROR)
+ << "Failed to initialize the start of the screen cast session.";
+ if (that->start_request_signal_id_) {
+ g_dbus_connection_signal_unsubscribe(connection,
+ that->start_request_signal_id_);
+ that->start_request_signal_id_ = 0;
+ }
2018-10-18 10:19:30 +00:00
+ that->portal_init_failed_ = true;
+ return;
+ }
+
+ g_free(handle);
+
+ LOG(LS_INFO) << "Subscribed to the start signal.";
+}
+
+// static
2018-10-18 10:19:30 +00:00
+void BaseCapturerPipeWire::OnStartRequestResponseSignal(
+ GDBusConnection* connection,
+ const gchar* sender_name,
+ const gchar* object_path,
+ const gchar* interface_name,
+ const gchar* signal_name,
+ GVariant* parameters,
+ gpointer user_data) {
2018-10-18 10:19:30 +00:00
+ BaseCapturerPipeWire* that =
+ static_cast<BaseCapturerPipeWire*>(user_data);
+ RTC_DCHECK(that);
+
+ LOG(LS_INFO) << "Start signal received.";
+ guint32 portal_response;
+ GVariant* response_data;
+ GVariantIter* iter = nullptr;
+ g_variant_get(parameters, "(u@a{sv})", &portal_response, &response_data);
+ if (portal_response || !response_data) {
+ LOG(LS_ERROR) << "Failed to start the screen cast session.";
2018-10-18 10:19:30 +00:00
+ that->portal_init_failed_ = true;
+ return;
+ }
+
+ // Array of PipeWire streams. See
+ // https://github.com/flatpak/xdg-desktop-portal/blob/master/data/org.freedesktop.portal.ScreenCast.xml
+ // documentation for <method name="Start">.
+ if (g_variant_lookup(response_data, "streams", "a(ua{sv})", &iter)) {
+ GVariant* variant;
+
+ while (g_variant_iter_next(iter, "@(ua{sv})", &variant)) {
+ guint32 stream_id;
+ gint32 width;
+ gint32 height;
+ GVariant* options;
+
+ g_variant_get(variant, "(u@a{sv})", &stream_id, &options);
+ RTC_DCHECK(options != nullptr);
+
+ g_variant_lookup(options, "size", "(ii)", &width, &height);
+
+ that->desktop_size_.set(width, height);
+
+ g_variant_unref(options);
+ g_variant_unref(variant);
+ }
+ }
+ g_variant_iter_free(iter);
+ g_variant_unref(response_data);
+
2018-10-18 10:19:30 +00:00
+ that->OpenPipeWireRemote();
+}
+
+void BaseCapturerPipeWire::OpenPipeWireRemote() {
+ GVariantBuilder builder;
+ g_variant_builder_init(&builder, G_VARIANT_TYPE_VARDICT);
+
+ LOG(LS_INFO) << "Opening the PipeWire remote.";
+
+ g_dbus_proxy_call_with_unix_fd_list(
+ proxy_, "OpenPipeWireRemote",
+ g_variant_new("(oa{sv})", session_handle_, &builder),
+ G_DBUS_CALL_FLAGS_NONE, /*timeout=*/-1, /*fd_list=*/nullptr,
+ /*cancellable=*/nullptr,
+ reinterpret_cast<GAsyncReadyCallback>(OnOpenPipeWireRemoteRequested),
+ this);
+}
+
+// static
+void BaseCapturerPipeWire::OnOpenPipeWireRemoteRequested(
+ GDBusConnection* connection,
+ GAsyncResult* result,
+ gpointer user_data) {
+ BaseCapturerPipeWire* that =
+ static_cast<BaseCapturerPipeWire*>(user_data);
+ RTC_DCHECK(that);
+
+ GError* error = nullptr;
+ GUnixFDList* outlist = nullptr;
+ GVariant* variant = g_dbus_proxy_call_with_unix_fd_list_finish(
+ that->proxy_, &outlist, result, &error);
+ if (!variant) {
+ LOG(LS_ERROR) << "Failed to open the PipeWire remote: "
+ << error->message;
+ g_error_free(error);
+ that->portal_init_failed_ = true;
+ return;
+ }
+
+ gint32 index;
+ g_variant_get(variant, "(h)", &index);
+
+ if ((that->pw_fd_ = g_unix_fd_list_get(outlist, index, &error)) == -1) {
+ LOG(LS_ERROR) << "Failed to get file descriptor from the list: "
+ << error->message;
+ g_error_free(error);
+ g_variant_unref(variant);
+ that->portal_init_failed_ = true;
+ return;
+ }
+
+ g_variant_unref(variant);
+ g_object_unref(outlist);
+
+ that->InitPipeWire();
+ LOG(LS_INFO) << "PipeWire remote opened.";
+}
+
+void BaseCapturerPipeWire::Start(Callback* callback) {
+ RTC_DCHECK(!callback_);
+ RTC_DCHECK(callback);
+
+ InitPortals();
+
+ callback_ = callback;
+}
+
+void BaseCapturerPipeWire::CaptureFrame() {
+ if (portal_init_failed_) {
+ callback_->OnCaptureResult(Result::ERROR_PERMANENT, nullptr);
+ return;
+ }
+
+ if (!current_frame_) {
+ callback_->OnCaptureResult(Result::ERROR_TEMPORARY, nullptr);
+ return;
+ }
+
+ std::unique_ptr<DesktopFrame> result(new BasicDesktopFrame(desktop_size_));
+ result->CopyPixelsFrom(
+ current_frame_, (desktop_size_.width() * kBytesPerPixelPw),
+ DesktopRect::MakeWH(desktop_size_.width(), desktop_size_.height()));
+ if (!result) {
+ callback_->OnCaptureResult(Result::ERROR_TEMPORARY, nullptr);
+ return;
+ }
+ callback_->OnCaptureResult(Result::SUCCESS, std::move(result));
+}
+
+bool BaseCapturerPipeWire::GetSourceList(SourceList* sources) {
+ RTC_DCHECK(sources->size() == 0);
+ // List of available screens is already presented by the xdg-desktop-portal.
+ // But we have to add an empty source as the code expects it.
+ sources->push_back({0});
+ return true;
+}
+
+bool BaseCapturerPipeWire::SelectSource(SourceId id) {
+ // Screen selection is handled by the xdg-desktop-portal.
+ return true;
+}
+
+} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.h.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.h
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.h.firefox-pipewire 2018-12-05 13:04:08.832063551 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.h 2018-12-05 13:04:08.832063551 +0100
2018-10-18 10:19:30 +00:00
@@ -0,0 +1,167 @@
+/*
+ * Copyright 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
+#ifndef MODULES_DESKTOP_CAPTURE_LINUX_BASE_CAPTURER_PIPEWIRE_H_
+#define MODULES_DESKTOP_CAPTURE_LINUX_BASE_CAPTURER_PIPEWIRE_H_
+
+#include <gio/gio.h>
+#define typeof __typeof__
+#include <pipewire/pipewire.h>
+#include <spa/param/video/format-utils.h>
+
+#include "webrtc/modules/desktop_capture/desktop_capture_options.h"
+#include "webrtc/modules/desktop_capture/desktop_capturer.h"
+
+namespace webrtc {
+
+class PipeWireType {
+ public:
+ spa_type_media_type media_type;
+ spa_type_media_subtype media_subtype;
+ spa_type_format_video format_video;
+ spa_type_video_format video_format;
+};
+
+class BaseCapturerPipeWire : public DesktopCapturer {
+ public:
+ enum CaptureSourceType { Screen = 1, Window };
+
+ explicit BaseCapturerPipeWire(CaptureSourceType source_type);
+ ~BaseCapturerPipeWire() override;
+
+ // DesktopCapturer interface.
+ void Start(Callback* delegate) override;
+ void Stop() override { callback_ = nullptr; }
+ void CaptureFrame() override;
+ bool GetSourceList(SourceList* sources) override;
+ bool SelectSource(SourceId id) override;
+
+ private:
+ // PipeWire types -->
+ pw_core* pw_core_ = nullptr;
+ pw_type* pw_core_type_ = nullptr;
+ pw_stream* pw_stream_ = nullptr;
+ pw_remote* pw_remote_ = nullptr;
+ pw_loop* pw_loop_ = nullptr;
+ pw_thread_loop* pw_main_loop_ = nullptr;
+ PipeWireType* pw_type_ = nullptr;
+
+ spa_hook spa_stream_listener_ = {};
+ spa_hook spa_remote_listener_ = {};
+
+ pw_stream_events pw_stream_events_ = {};
+ pw_remote_events pw_remote_events_ = {};
+
+ spa_video_info_raw* spa_video_format_ = nullptr;
+
+ gint32 pw_fd_ = -1;
+
+ CaptureSourceType capture_source_type_ =
+ BaseCapturerPipeWire::CaptureSourceType::Screen;
+
+ // <-- end of PipeWire types
+
+ GDBusConnection* connection_ = nullptr;
+ GDBusProxy* proxy_ = nullptr;
+ gchar* portal_handle_ = nullptr;
+ gchar* session_handle_ = nullptr;
+ gchar* sources_handle_ = nullptr;
+ gchar* start_handle_ = nullptr;
+ guint session_request_signal_id_ = 0;
+ guint sources_request_signal_id_ = 0;
+ guint start_request_signal_id_ = 0;
+
+ DesktopSize desktop_size_ = {};
+ DesktopCaptureOptions options_ = {};
+
+ uint8_t* current_frame_ = nullptr;
+ Callback* callback_ = nullptr;
+
+ bool portal_init_failed_ = false;
+
+ void InitPortals();
+ void InitPipeWire();
+ void InitPipeWireTypes();
+
+ void CreateReceivingStream();
+ void HandleBuffer(pw_buffer* buffer);
+
+ void ConvertRGBxToBGRx(uint8_t* frame, uint32_t size);
+
+ static void OnStateChanged(void* data,
+ pw_remote_state old_state,
+ pw_remote_state state,
+ const char* error);
+ static void OnStreamStateChanged(void* data,
+ pw_stream_state old_state,
+ pw_stream_state state,
+ const char* error_message);
+
+ static void OnStreamFormatChanged(void* data, const struct spa_pod* format);
+ static void OnStreamProcess(void* data);
+ static void OnNewBuffer(void* data, uint32_t id);
+
+ guint SetupRequestResponseSignal(const gchar* object_path,
+ GDBusSignalCallback callback);
+
+ static void OnProxyRequested(GObject* object,
+ GAsyncResult* result,
+ gpointer user_data);
+
+ static gchar* PrepareSignalHandle(GDBusConnection* connection,
+ const gchar* token);
+
+ void SessionRequest();
+ static void OnSessionRequested(GDBusConnection* connection,
+ GAsyncResult* result,
+ gpointer user_data);
+ static void OnSessionRequestResponseSignal(GDBusConnection* connection,
+ const gchar* sender_name,
+ const gchar* object_path,
+ const gchar* interface_name,
+ const gchar* signal_name,
+ GVariant* parameters,
+ gpointer user_data);
+
+ void SourcesRequest();
+ static void OnSourcesRequested(GDBusConnection* connection,
+ GAsyncResult* result,
+ gpointer user_data);
+ static void OnSourcesRequestResponseSignal(GDBusConnection* connection,
+ const gchar* sender_name,
+ const gchar* object_path,
+ const gchar* interface_name,
+ const gchar* signal_name,
+ GVariant* parameters,
+ gpointer user_data);
+
+ void StartRequest();
+ static void OnStartRequested(GDBusConnection* connection,
+ GAsyncResult* result,
+ gpointer user_data);
+ static void OnStartRequestResponseSignal(GDBusConnection* connection,
+ const gchar* sender_name,
+ const gchar* object_path,
+ const gchar* interface_name,
+ const gchar* signal_name,
+ GVariant* parameters,
+ gpointer user_data);
+
+ void OpenPipeWireRemote();
+ static void OnOpenPipeWireRemoteRequested(GDBusConnection* connection,
+ GAsyncResult* result,
+ gpointer user_data);
+
+ RTC_DISALLOW_COPY_AND_ASSIGN(BaseCapturerPipeWire);
+};
+
+} // namespace webrtc
+
+#endif // MODULES_DESKTOP_CAPTURE_LINUX_BASE_CAPTURER_PIPEWIRE_H_
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/BUILD.gn.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/BUILD.gn
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/BUILD.gn.firefox-pipewire 2018-12-04 00:34:14.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/BUILD.gn 2018-12-05 13:04:08.828063572 +0100
@@ -213,7 +213,6 @@ rtc_static_library("desktop_capture") {
if (build_with_mozilla) {
sources += [
"app_capturer_x11.cc",
- "app_capturer_x11.h",
"x11/desktop_device_info_x11.cc",
"x11/desktop_device_info_x11.h",
"x11/shared_x_util.cc",
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capture_gn/moz.build.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capture_gn/moz.build
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capture_gn/moz.build.firefox-pipewire 2018-12-05 13:04:08.832063551 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capture_gn/moz.build 2018-12-05 13:12:40.063396138 +0100
@@ -100,10 +100,6 @@ if CONFIG["OS_TARGET"] == "DragonFly":
]
UNIFIED_SOURCES += [
- "/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_x11.cc",
- "/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_x11.cc",
- "/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_x11.cc",
- "/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_x11.cc",
"/media/webrtc/trunk/webrtc/modules/desktop_capture/x11/desktop_device_info_x11.cc",
"/media/webrtc/trunk/webrtc/modules/desktop_capture/x11/shared_x_display.cc",
"/media/webrtc/trunk/webrtc/modules/desktop_capture/x11/shared_x_util.cc",
@@ -111,6 +107,38 @@ if CONFIG["OS_TARGET"] == "DragonFly":
"/media/webrtc/trunk/webrtc/modules/desktop_capture/x11/x_server_pixel_buffer.cc"
]
+# Common stuff between Pipewire and X11
+if CONFIG["OS_TARGET"] == "Linux":
+
+ UNIFIED_SOURCES += [
+ "/media/webrtc/trunk/webrtc/modules/desktop_capture/app_capturer_linux.cc",
+ "/media/webrtc/trunk/webrtc/modules/desktop_capture/base_capturer_pipewire.cc",
+ "/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_linux.cc",
+ "/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_linux.cc",
+ "/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.cc",
+ "/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_linux.cc",
+ "/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.cc",
+ ]
+
+# Pipewire
+if CONFIG["OS_TARGET"] == "Linux":
+
+ DEFINES["USE_PIPEWIRE"] = "1"
+
+ OS_LIBS += [
+ "rt",
+ "pipewire-0.2",
+ "glib-2.0",
+ "gio-2.0",
+ "gobject-2.0"
+ ]
+
+ CXXFLAGS += CONFIG['TK_CFLAGS']
+
+ UNIFIED_SOURCES += [
+ "/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_null.cc"
+ ]
+
if CONFIG["OS_TARGET"] == "FreeBSD":
DEFINES["USE_X11"] = "1"
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capturer.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capturer.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capturer.cc.firefox-pipewire 2018-12-04 00:34:14.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capturer.cc 2018-12-05 13:04:08.832063551 +0100
@@ -66,4 +66,17 @@ std::unique_ptr<DesktopCapturer> Desktop
2018-10-18 10:19:30 +00:00
return capturer;
}
+#if defined(USE_PIPEWIRE) || defined(USE_X11)
+bool DesktopCapturer::IsRunningUnderWayland() {
+ const char* xdg_session_type = getenv("XDG_SESSION_TYPE");
+ if (!xdg_session_type || strncmp(xdg_session_type, "wayland", 7) != 0)
+ return false;
+
+ if (!(getenv("WAYLAND_DISPLAY")))
+ return false;
+
+ return true;
+}
+#endif
+
} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capturer.h.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capturer.h
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capturer.h.firefox-pipewire 2018-12-04 00:34:14.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/desktop_capturer.h 2018-12-05 13:04:08.832063551 +0100
2018-10-18 10:19:30 +00:00
@@ -129,6 +129,10 @@ class DesktopCapturer {
static std::unique_ptr<DesktopCapturer> CreateAppCapturer(
const DesktopCaptureOptions& options);
+#if defined(USE_PIPEWIRE) || defined(USE_X11)
+ static bool IsRunningUnderWayland();
+#endif
+
protected:
// CroppingWindowCapturer needs to create raw capturers without wrappers, so
// the following two functions are protected.
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_linux.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_linux.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_linux.cc.firefox-pipewire 2018-12-05 13:04:08.832063551 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_linux.cc 2018-12-05 13:04:08.832063551 +0100
2018-10-18 10:19:30 +00:00
@@ -0,0 +1,44 @@
+/*
+ * Copyright (c) 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
+#include "webrtc/modules/desktop_capture/mouse_cursor_monitor.h"
+
+#if defined(USE_PIPEWIRE)
+#include "webrtc/modules/desktop_capture/mouse_cursor_monitor_null.cc"
+#endif // defined(USE_PIPEWIRE)
+
+#if defined(USE_X11)
+#include "webrtc/modules/desktop_capture/mouse_cursor_monitor_x11.cc"
+#endif // defined(USE_X11)
+
+namespace webrtc {
+
+// static
+MouseCursorMonitor* MouseCursorMonitor::CreateForWindow(
+ const DesktopCaptureOptions& options, WindowId window) {
+#if defined(USE_X11)
+ return MouseCursorMonitorX11::CreateForWindow(options, window);
+#else
+ return nullptr;
+#endif // defined(USE_X11)
+}
+
+// static
+MouseCursorMonitor* MouseCursorMonitor::CreateForScreen(
+ const DesktopCaptureOptions& options,
+ ScreenId screen) {
+#if defined(USE_X11)
+ return MouseCursorMonitorX11::CreateForScreen(options, screen);
+#else
+ return nullptr;
+#endif // defined(USE_X11)
+}
+
+} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_null.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_null.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_null.cc.firefox-pipewire 2018-12-04 00:34:15.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_null.cc 2018-12-05 13:04:08.832063551 +0100
2018-10-18 10:19:30 +00:00
@@ -14,6 +14,7 @@
namespace webrtc {
+#if not defined(USE_PIPEWIRE)
MouseCursorMonitor* MouseCursorMonitor::CreateForWindow(
const DesktopCaptureOptions& options,
WindowId window) {
@@ -25,5 +26,6 @@ MouseCursorMonitor* MouseCursorMonitor::
2018-10-18 10:19:30 +00:00
ScreenId screen) {
return NULL;
}
+#endif // not defined(USE_PIPEWIRE)
} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_x11.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_x11.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_x11.cc.firefox-pipewire 2018-12-04 00:34:14.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/mouse_cursor_monitor_x11.cc 2018-12-05 13:04:08.832063551 +0100
@@ -69,6 +69,9 @@ class MouseCursorMonitorX11 : public Mou
2018-10-18 10:19:30 +00:00
void Stop() override;
void Capture() override;
+ static MouseCursorMonitor* CreateForWindow(const DesktopCaptureOptions& options, WindowId window);
+ static MouseCursorMonitor* CreateForScreen(const DesktopCaptureOptions& options, ScreenId screen);
+
private:
// SharedXDisplay::XEventHandler interface.
bool HandleXEvent(const XEvent& event) override;
@@ -244,7 +247,7 @@ void MouseCursorMonitorX11::CaptureCurso
2018-10-18 10:19:30 +00:00
}
// static
-MouseCursorMonitor* MouseCursorMonitor::CreateForWindow(
+MouseCursorMonitor* MouseCursorMonitorX11::CreateForWindow(
const DesktopCaptureOptions& options, WindowId window) {
if (!options.x_display())
return NULL;
@@ -254,7 +257,7 @@ MouseCursorMonitor* MouseCursorMonitor::
2018-10-18 10:19:30 +00:00
return new MouseCursorMonitorX11(options, outer_window, window);
}
-MouseCursorMonitor* MouseCursorMonitor::CreateForScreen(
+MouseCursorMonitor* MouseCursorMonitorX11::CreateForScreen(
const DesktopCaptureOptions& options,
ScreenId screen) {
if (!options.x_display())
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_linux.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_linux.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_linux.cc.firefox-pipewire 2018-12-05 13:04:08.833063546 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_linux.cc 2018-12-05 13:04:08.833063546 +0100
2018-10-18 10:19:30 +00:00
@@ -0,0 +1,40 @@
+/*
+ * Copyright 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
+#include "webrtc/modules/desktop_capture/desktop_capture_options.h"
+#include "webrtc/modules/desktop_capture/desktop_capturer.h"
+
2018-10-18 10:19:30 +00:00
+#if defined(USE_PIPEWIRE)
+#include "webrtc/modules/desktop_capture/screen_capturer_pipewire.h"
+#endif // defined(USE_PIPEWIRE)
+
2018-10-18 10:19:30 +00:00
+#if defined(USE_X11)
+#include "webrtc/modules/desktop_capture/screen_capturer_x11.cc"
+#endif // defined(USE_X11)
+
2018-10-18 10:19:30 +00:00
+namespace webrtc {
+
+// static
2018-10-18 10:19:30 +00:00
+std::unique_ptr<DesktopCapturer> DesktopCapturer::CreateRawScreenCapturer(
+ const DesktopCaptureOptions& options) {
+#if defined(USE_PIPEWIRE)
+ if (DesktopCapturer::IsRunningUnderWayland()) {
+ return ScreenCapturerPipeWire::CreateRawScreenCapturer(options);
+ }
2018-10-18 10:19:30 +00:00
+#endif // defined(USE_PIPEWIRE)
+
2018-10-18 10:19:30 +00:00
+#if defined(USE_X11)
+ return ScreenCapturerX11::CreateRawScreenCapturer(options);
+#endif // defined(USE_X11)
+
2018-10-18 10:19:30 +00:00
+ return nullptr;
+}
+
2018-10-18 10:19:30 +00:00
+} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.cc.firefox-pipewire 2018-12-05 13:04:08.833063546 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.cc 2018-12-05 13:04:08.833063546 +0100
2018-10-18 10:19:30 +00:00
@@ -0,0 +1,31 @@
+/*
+ * Copyright 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
2018-10-18 10:19:30 +00:00
+#include "webrtc/modules/desktop_capture/screen_capturer_pipewire.h"
+
2018-10-18 10:19:30 +00:00
+#include <memory>
+
2018-10-18 10:19:30 +00:00
+namespace webrtc {
+
2018-10-18 10:19:30 +00:00
+ScreenCapturerPipeWire::ScreenCapturerPipeWire()
+ : BaseCapturerPipeWire(BaseCapturerPipeWire::CaptureSourceType::Screen) {}
+ScreenCapturerPipeWire::~ScreenCapturerPipeWire() {}
+
+// static
+std::unique_ptr<DesktopCapturer>
+ScreenCapturerPipeWire::CreateRawScreenCapturer(
+ const DesktopCaptureOptions& options) {
+ std::unique_ptr<ScreenCapturerPipeWire> capturer(
+ new ScreenCapturerPipeWire());
+
+ return std::move(capturer);
+}
+
+} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.h.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.h
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.h.firefox-pipewire 2018-12-05 13:04:08.833063546 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_pipewire.h 2018-12-05 13:04:08.833063546 +0100
2018-10-18 10:19:30 +00:00
@@ -0,0 +1,33 @@
+/*
+ * Copyright 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
+#ifndef MODULES_DESKTOP_CAPTURE_LINUX_SCREEN_CAPTURER_PIPEWIRE_H_
+#define MODULES_DESKTOP_CAPTURE_LINUX_SCREEN_CAPTURER_PIPEWIRE_H_
+
+#include <memory>
+
+#include "webrtc/modules/desktop_capture/base_capturer_pipewire.h"
+
+namespace webrtc {
+
+class ScreenCapturerPipeWire : public BaseCapturerPipeWire {
+ public:
+ ScreenCapturerPipeWire();
+ ~ScreenCapturerPipeWire() override;
+
+ static std::unique_ptr<DesktopCapturer> CreateRawScreenCapturer(
+ const DesktopCaptureOptions& options);
+
+ RTC_DISALLOW_COPY_AND_ASSIGN(ScreenCapturerPipeWire);
+};
+
+} // namespace webrtc
+
+#endif // MODULES_DESKTOP_CAPTURE_LINUX_SCREEN_CAPTURER_PIPEWIRE_H_
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_linux.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_linux.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_linux.cc.firefox-pipewire 2018-12-05 13:04:08.833063546 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_linux.cc 2018-12-05 13:04:08.833063546 +0100
@@ -0,0 +1,40 @@
+/*
2018-10-18 10:19:30 +00:00
+ * Copyright 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
+#include "webrtc/modules/desktop_capture/desktop_capture_options.h"
+#include "webrtc/modules/desktop_capture/desktop_capturer.h"
+
+#if defined(USE_PIPEWIRE)
2018-10-18 10:19:30 +00:00
+#include "webrtc/modules/desktop_capture/window_capturer_pipewire.h"
+#endif // defined(USE_PIPEWIRE)
+
+#if defined(USE_X11)
+#include "webrtc/modules/desktop_capture/window_capturer_x11.cc"
2018-10-18 10:19:30 +00:00
+#endif // defined(USE_X11)
+
+namespace webrtc {
+
+// static
+std::unique_ptr<DesktopCapturer> DesktopCapturer::CreateRawWindowCapturer(
+ const DesktopCaptureOptions& options) {
+#if defined(USE_PIPEWIRE)
+ if (DesktopCapturer::IsRunningUnderWayland()) {
2018-10-18 10:19:30 +00:00
+ return WindowCapturerPipeWire::CreateRawWindowCapturer(options);
+ }
2018-10-18 10:19:30 +00:00
+#endif // defined(USE_PIPEWIRE)
+
+#if defined(USE_X11)
+ return WindowCapturerX11::CreateRawWindowCapturer(options);
2018-10-18 10:19:30 +00:00
+#endif // defined(USE_X11)
+
+ return nullptr;
+}
+
+} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_null.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_null.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_null.cc.firefox-pipewire 2018-12-04 00:34:14.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_null.cc 2018-12-05 13:04:08.833063546 +0100
@@ -67,10 +67,12 @@ void WindowCapturerNull::CaptureFrame()
2018-10-18 10:19:30 +00:00
} // namespace
2018-10-18 10:19:30 +00:00
+#if not defined(USE_PIPEWIRE)
// static
std::unique_ptr<DesktopCapturer> DesktopCapturer::CreateRawWindowCapturer(
const DesktopCaptureOptions& options) {
return std::unique_ptr<DesktopCapturer>(new WindowCapturerNull());
}
+#endif // not defined(USE_PIPEWIRE)
2018-10-18 10:19:30 +00:00
} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.cc.firefox-pipewire 2018-12-05 13:04:08.833063546 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.cc 2018-12-05 13:04:08.833063546 +0100
2018-10-18 10:19:30 +00:00
@@ -0,0 +1,28 @@
+/*
+ * Copyright 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
+#include "webrtc/modules/desktop_capture/window_capturer_pipewire.h"
+
+namespace webrtc {
+
+WindowCapturerPipeWire::WindowCapturerPipeWire()
+ : BaseCapturerPipeWire(BaseCapturerPipeWire::CaptureSourceType::Window) {}
+WindowCapturerPipeWire::~WindowCapturerPipeWire() {}
+
+// static
+std::unique_ptr<DesktopCapturer>
+WindowCapturerPipeWire::CreateRawWindowCapturer(
+ const DesktopCaptureOptions& options) {
+ std::unique_ptr<WindowCapturerPipeWire> capturer(
+ new WindowCapturerPipeWire());
+
+ return std::move(capturer);
+}
+} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.h.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.h
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.h.firefox-pipewire 2018-12-05 13:04:08.833063546 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_pipewire.h 2018-12-05 13:04:08.833063546 +0100
2018-10-18 10:19:30 +00:00
@@ -0,0 +1,33 @@
+/*
+ * Copyright 2018 The WebRTC project authors. All Rights Reserved.
+ *
+ * Use of this source code is governed by a BSD-style license
+ * that can be found in the LICENSE file in the root of the source
+ * tree. An additional intellectual property rights grant can be found
+ * in the file PATENTS. All contributing project authors may
+ * be found in the AUTHORS file in the root of the source tree.
+ */
+
+#ifndef MODULES_DESKTOP_CAPTURE_LINUX_WINDOW_CAPTURER_PIPEWIRE_H_
+#define MODULES_DESKTOP_CAPTURE_LINUX_WINDOW_CAPTURER_PIPEWIRE_H_
+
+#include <memory>
+
+#include "webrtc/modules/desktop_capture/base_capturer_pipewire.h"
+
+namespace webrtc {
+
+class WindowCapturerPipeWire : public BaseCapturerPipeWire {
+ public:
+ WindowCapturerPipeWire();
+ ~WindowCapturerPipeWire() override;
+
+ static std::unique_ptr<DesktopCapturer> CreateRawWindowCapturer(
+ const DesktopCaptureOptions& options);
+
+ RTC_DISALLOW_COPY_AND_ASSIGN(WindowCapturerPipeWire);
+};
+
+} // namespace webrtc
+
+#endif // MODULES_DESKTOP_CAPTURE_LINUX_WINDOW_CAPTURER_PIPEWIRE_H_
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_x11.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_x11.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_x11.cc.firefox-pipewire 2018-12-04 00:34:14.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/window_capturer_x11.cc 2018-12-05 13:04:08.833063546 +0100
@@ -32,11 +32,13 @@ namespace webrtc {
2018-10-18 10:19:30 +00:00
namespace {
2018-10-18 10:19:30 +00:00
-class WindowCapturerLinux : public DesktopCapturer,
+class WindowCapturerX11 : public DesktopCapturer,
public SharedXDisplay::XEventHandler {
public:
- WindowCapturerLinux(const DesktopCaptureOptions& options);
- ~WindowCapturerLinux() override;
+ WindowCapturerX11(const DesktopCaptureOptions& options);
+ ~WindowCapturerX11() override;
+
+ static std::unique_ptr<DesktopCapturer> CreateRawWindowCapturer(const DesktopCaptureOptions& options);
2018-10-18 10:19:30 +00:00
// DesktopCapturer interface.
void Start(Callback* callback) override;
@@ -78,10 +80,10 @@ class WindowCapturerLinux : public Deskt
::Window selected_window_ = 0;
XServerPixelBuffer x_server_pixel_buffer_;
2018-10-18 10:19:30 +00:00
- RTC_DISALLOW_COPY_AND_ASSIGN(WindowCapturerLinux);
+ RTC_DISALLOW_COPY_AND_ASSIGN(WindowCapturerX11);
};
2018-10-18 10:19:30 +00:00
-WindowCapturerLinux::WindowCapturerLinux(const DesktopCaptureOptions& options)
+WindowCapturerX11::WindowCapturerX11(const DesktopCaptureOptions& options)
: x_display_(options.x_display()) {
// Create Atoms so we don't need to do it every time they are used.
wm_state_atom_ = XInternAtom(display(), "WM_STATE", True);
@@ -102,11 +104,11 @@ WindowCapturerLinux::WindowCapturerLinux
x_display_->AddEventHandler(ConfigureNotify, this);
}
2018-10-18 10:19:30 +00:00
-WindowCapturerLinux::~WindowCapturerLinux() {
+WindowCapturerX11::~WindowCapturerX11() {
x_display_->RemoveEventHandler(ConfigureNotify, this);
}
2018-10-18 10:19:30 +00:00
-bool WindowCapturerLinux::GetSourceList(SourceList* sources) {
+bool WindowCapturerX11::GetSourceList(SourceList* sources) {
SourceList result;
2018-10-18 10:19:30 +00:00
XErrorTrap error_trap(display());
@@ -159,7 +161,7 @@ bool WindowCapturerLinux::GetSourceList(
return true;
}
2018-10-18 10:19:30 +00:00
-bool WindowCapturerLinux::SelectSource(SourceId id) {
+bool WindowCapturerX11::SelectSource(SourceId id) {
if (!x_server_pixel_buffer_.Init(display(), id))
return false;
2018-10-18 10:19:30 +00:00
@@ -180,7 +182,7 @@ bool WindowCapturerLinux::SelectSource(S
return true;
}
2018-10-18 10:19:30 +00:00
-bool WindowCapturerLinux::FocusOnSelectedSource() {
+bool WindowCapturerX11::FocusOnSelectedSource() {
if (!selected_window_)
return false;
2018-10-18 10:19:30 +00:00
@@ -229,18 +231,18 @@ bool WindowCapturerLinux::FocusOnSelecte
return true;
}
2018-10-18 10:19:30 +00:00
-void WindowCapturerLinux::Start(Callback* callback) {
+void WindowCapturerX11::Start(Callback* callback) {
assert(!callback_);
assert(callback);
2018-10-18 10:19:30 +00:00
callback_ = callback;
}
2018-10-18 10:19:30 +00:00
-void WindowCapturerLinux::Stop() {
+void WindowCapturerX11::Stop() {
callback_ = NULL;
}
2018-10-18 10:19:30 +00:00
-void WindowCapturerLinux::CaptureFrame() {
+void WindowCapturerX11::CaptureFrame() {
x_display_->ProcessPendingXEvents();
2018-10-18 10:19:30 +00:00
if (!x_server_pixel_buffer_.IsWindowValid()) {
@@ -274,7 +276,7 @@ void WindowCapturerLinux::CaptureFrame()
callback_->OnCaptureResult(Result::SUCCESS, std::move(frame));
}
2018-10-18 10:19:30 +00:00
-bool WindowCapturerLinux::HandleXEvent(const XEvent& event) {
+bool WindowCapturerX11::HandleXEvent(const XEvent& event) {
if (event.type == ConfigureNotify) {
XConfigureEvent xce = event.xconfigure;
if (!DesktopSize(xce.width, xce.height).equals(
@@ -288,7 +290,7 @@ bool WindowCapturerLinux::HandleXEvent(c
return false;
}
2018-10-18 10:19:30 +00:00
-::Window WindowCapturerLinux::GetApplicationWindow(::Window window) {
+::Window WindowCapturerX11::GetApplicationWindow(::Window window) {
// Get WM_STATE property of the window.
XWindowProperty<uint32_t> window_state(display(), window, wm_state_atom_);
2018-10-18 10:19:30 +00:00
@@ -326,7 +328,7 @@ bool WindowCapturerLinux::HandleXEvent(c
return app_window;
}
2018-10-18 10:19:30 +00:00
-bool WindowCapturerLinux::IsDesktopElement(::Window window) {
+bool WindowCapturerX11::IsDesktopElement(::Window window) {
if (window == 0)
return false;
2018-10-18 10:19:30 +00:00
@@ -361,7 +363,7 @@ bool WindowCapturerLinux::IsDesktopEleme
return result;
}
2018-10-18 10:19:30 +00:00
-bool WindowCapturerLinux::GetWindowTitle(::Window window, std::string* title) {
+bool WindowCapturerX11::GetWindowTitle(::Window window, std::string* title) {
int status;
bool result = false;
XTextProperty window_name;
@@ -392,7 +394,7 @@ bool WindowCapturerLinux::GetWindowTitle
2018-10-18 10:19:30 +00:00
} // namespace
2018-10-18 10:19:30 +00:00
-int WindowCapturerLinux::GetWindowProcessID(::Window window) {
+int WindowCapturerX11::GetWindowProcessID(::Window window) {
// Get _NET_WM_PID property of the window.
Atom process_atom = XInternAtom(display(), "_NET_WM_PID", True);
XWindowProperty<uint32_t> process_id(display(), window, process_atom);
@@ -401,11 +403,11 @@ int WindowCapturerLinux::GetWindowProces
}
2018-10-18 10:19:30 +00:00
// static
-std::unique_ptr<DesktopCapturer> DesktopCapturer::CreateRawWindowCapturer(
+std::unique_ptr<DesktopCapturer> WindowCapturerX11::CreateRawWindowCapturer(
const DesktopCaptureOptions& options) {
if (!options.x_display())
return nullptr;
- return std::unique_ptr<DesktopCapturer>(new WindowCapturerLinux(options));
+ return std::unique_ptr<DesktopCapturer>(new WindowCapturerX11(options));
}
2018-10-18 10:19:30 +00:00
} // namespace webrtc
diff -up firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_x11.cc.firefox-pipewire firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_x11.cc
--- firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_x11.cc.firefox-pipewire 2018-12-04 00:34:14.000000000 +0100
+++ firefox-64.0/media/webrtc/trunk/webrtc/modules/desktop_capture/screen_capturer_x11.cc 2018-12-05 13:04:08.833063546 +0100
@@ -34,18 +34,20 @@
namespace webrtc {
namespace {
2018-10-18 10:19:30 +00:00
// A class to perform video frame capturing for Linux.
//
// If XDamage is used, this class sets DesktopFrame::updated_region() according
// to the areas reported by XDamage. Otherwise this class does not detect
// DesktopFrame::updated_region(), the field is always set to the entire frame
// rectangle. ScreenCapturerDifferWrapper should be used if that functionality
// is necessary.
-class ScreenCapturerLinux : public DesktopCapturer,
+class ScreenCapturerX11 : public DesktopCapturer,
public SharedXDisplay::XEventHandler {
public:
- ScreenCapturerLinux();
- ~ScreenCapturerLinux() override;
+ ScreenCapturerX11();
+ ~ScreenCapturerX11() override;
+
+ static std::unique_ptr<DesktopCapturer> CreateRawScreenCapturer(const DesktopCaptureOptions& options);
2018-10-18 10:19:30 +00:00
// TODO(ajwong): Do we really want this to be synchronous?
bool Init(const DesktopCaptureOptions& options);
@@ -118,14 +120,13 @@ class ScreenCapturerLinux : public Deskt
// current with the last buffer used.
DesktopRegion last_invalid_region_;
2018-10-18 10:19:30 +00:00
- RTC_DISALLOW_COPY_AND_ASSIGN(ScreenCapturerLinux);
+ RTC_DISALLOW_COPY_AND_ASSIGN(ScreenCapturerX11);
};
-
-ScreenCapturerLinux::ScreenCapturerLinux() {
+ScreenCapturerX11::ScreenCapturerX11() {
helper_.SetLogGridSize(4);
}
2018-10-18 10:19:30 +00:00
-ScreenCapturerLinux::~ScreenCapturerLinux() {
+ScreenCapturerX11::~ScreenCapturerX11() {
options_.x_display()->RemoveEventHandler(ConfigureNotify, this);
if (use_damage_) {
options_.x_display()->RemoveEventHandler(
@@ -134,7 +135,7 @@ ScreenCapturerLinux::~ScreenCapturerLinu
DeinitXlib();
}
2018-10-18 10:19:30 +00:00
-bool ScreenCapturerLinux::Init(const DesktopCaptureOptions& options) {
+bool ScreenCapturerX11::Init(const DesktopCaptureOptions& options) {
options_ = options;
2018-10-18 10:19:30 +00:00
root_window_ = RootWindow(display(), DefaultScreen(display()));
@@ -177,7 +178,7 @@ bool ScreenCapturerLinux::Init(const Des
return true;
}
2018-10-18 10:19:30 +00:00
-void ScreenCapturerLinux::InitXDamage() {
+void ScreenCapturerX11::InitXDamage() {
// Our use of XDamage requires XFixes.
if (!has_xfixes_) {
return;
@@ -218,18 +219,18 @@ void ScreenCapturerLinux::InitXDamage()
LOG(LS_INFO) << "Using XDamage extension.";
}
2018-10-18 10:19:30 +00:00
-void ScreenCapturerLinux::Start(Callback* callback) {
+void ScreenCapturerX11::Start(Callback* callback) {
RTC_DCHECK(!callback_);
RTC_DCHECK(callback);
2018-10-18 10:19:30 +00:00
callback_ = callback;
}
2018-10-18 10:19:30 +00:00
-void ScreenCapturerLinux::Stop() {
+void ScreenCapturerX11::Stop() {
callback_ = NULL;
}
2018-10-18 10:19:30 +00:00
-void ScreenCapturerLinux::CaptureFrame() {
+void ScreenCapturerX11::CaptureFrame() {
int64_t capture_start_time_nanos = rtc::TimeNanos();
2018-10-18 10:19:30 +00:00
queue_.MoveToNextFrame();
@@ -243,6 +244,7 @@ void ScreenCapturerLinux::CaptureFrame()
// in a good shape.
if (!x_server_pixel_buffer_.is_initialized()) {
// We failed to initialize pixel buffer.
+ LOG(LS_ERROR) << "Pixel buffer is not initialized.";
callback_->OnCaptureResult(Result::ERROR_PERMANENT, nullptr);
return;
}
@@ -258,6 +260,7 @@ void ScreenCapturerLinux::CaptureFrame()
2018-10-18 10:19:30 +00:00
std::unique_ptr<DesktopFrame> result = CaptureScreen();
if (!result) {
+ LOG(LS_WARNING) << "Temporarily failed to capture screen.";
callback_->OnCaptureResult(Result::ERROR_TEMPORARY, nullptr);
return;
}
@@ -268,19 +271,19 @@ void ScreenCapturerLinux::CaptureFrame()
callback_->OnCaptureResult(Result::SUCCESS, std::move(result));
}
2018-10-18 10:19:30 +00:00
-bool ScreenCapturerLinux::GetSourceList(SourceList* sources) {
+bool ScreenCapturerX11::GetSourceList(SourceList* sources) {
RTC_DCHECK(sources->size() == 0);
// TODO(jiayl): implement screen enumeration.
sources->push_back({0});
return true;
}
2018-10-18 10:19:30 +00:00
-bool ScreenCapturerLinux::SelectSource(SourceId id) {
+bool ScreenCapturerX11::SelectSource(SourceId id) {
// TODO(jiayl): implement screen selection.
return true;
}
2018-10-18 10:19:30 +00:00
-bool ScreenCapturerLinux::HandleXEvent(const XEvent& event) {
+bool ScreenCapturerX11::HandleXEvent(const XEvent& event) {
if (use_damage_ && (event.type == damage_event_base_ + XDamageNotify)) {
const XDamageNotifyEvent* damage_event =
reinterpret_cast<const XDamageNotifyEvent*>(&event);
@@ -295,7 +298,7 @@ bool ScreenCapturerLinux::HandleXEvent(c
return false;
}
2018-10-18 10:19:30 +00:00
-std::unique_ptr<DesktopFrame> ScreenCapturerLinux::CaptureScreen() {
+std::unique_ptr<DesktopFrame> ScreenCapturerX11::CaptureScreen() {
std::unique_ptr<SharedDesktopFrame> frame = queue_.current_frame()->Share();
RTC_DCHECK(x_server_pixel_buffer_.window_size().equals(frame->size()));
2018-10-18 10:19:30 +00:00
@@ -345,25 +348,26 @@ std::unique_ptr<DesktopFrame> ScreenCapt
// Doing full-screen polling, or this is the first capture after a
// screen-resolution change. In either case, need a full-screen capture.
DesktopRect screen_rect = DesktopRect::MakeSize(frame->size());
- x_server_pixel_buffer_.CaptureRect(screen_rect, frame.get());
+ if (!x_server_pixel_buffer_.CaptureRect(screen_rect, frame.get()))
+ return nullptr;
updated_region->SetRect(screen_rect);
}
2018-10-18 10:19:30 +00:00
return std::move(frame);
}
2018-10-18 10:19:30 +00:00
-void ScreenCapturerLinux::ScreenConfigurationChanged() {
+void ScreenCapturerX11::ScreenConfigurationChanged() {
// Make sure the frame buffers will be reallocated.
queue_.Reset();
2018-10-18 10:19:30 +00:00
helper_.ClearInvalidRegion();
if (!x_server_pixel_buffer_.Init(display(), DefaultRootWindow(display()))) {
LOG(LS_ERROR) << "Failed to initialize pixel buffer after screen "
- "configuration change.";
+ "configuration change.";
}
}
2018-10-18 10:19:30 +00:00
-void ScreenCapturerLinux::SynchronizeFrame() {
+void ScreenCapturerX11::SynchronizeFrame() {
// Synchronize the current buffer with the previous one since we do not
// capture the entire desktop. Note that encoder may be reading from the
// previous buffer at this time so thread access complaints are false
@@ -383,7 +387,7 @@ void ScreenCapturerLinux::SynchronizeFra
}
}
2018-10-18 10:19:30 +00:00
-void ScreenCapturerLinux::DeinitXlib() {
+void ScreenCapturerX11::DeinitXlib() {
if (gc_) {
XFreeGC(display(), gc_);
gc_ = nullptr;
@@ -407,17 +411,17 @@ void ScreenCapturerLinux::DeinitXlib() {
} // namespace
2018-10-18 10:19:30 +00:00
// static
-std::unique_ptr<DesktopCapturer> DesktopCapturer::CreateRawScreenCapturer(
+std::unique_ptr<DesktopCapturer> ScreenCapturerX11::CreateRawScreenCapturer(
const DesktopCaptureOptions& options) {
if (!options.x_display())
return nullptr;
2018-10-18 10:19:30 +00:00
- std::unique_ptr<ScreenCapturerLinux> capturer(new ScreenCapturerLinux());
+ std::unique_ptr<ScreenCapturerX11> capturer(new ScreenCapturerX11());
if (!capturer.get()->Init(options)) {
return nullptr;
}
2018-10-18 10:19:30 +00:00
- return std::unique_ptr<DesktopCapturer>(capturer.release());
+ return std::move(capturer);
}
2018-10-18 10:19:30 +00:00
} // namespace webrtc
@@ -34,18 +34,20 @@
namespace {
2018-10-18 10:19:30 +00:00
-// A class to perform video frame capturing for Linux.
+// A class to perform video frame capturing for Linux on X11.
//
// If XDamage is used, this class sets DesktopFrame::updated_region() according